import { rename_ids } from '../../../repeater/frontend/src/utilities';

(function ($){
    const CLONED_FIELD_INDICATOR_CLASS = 'cfr-clone';

    const handleMagicTag = (fieldWrapper, tagValues) => {
        const label = fieldWrapper.find('label.form-control-label');

        for( const [tagName, value] of Object.entries( tagValues ) ) {
            label.text(label.text().replace(`{${tagName}}`, value));
        }
    }

    const clone = (originEl, originGroup, fieldWrapper, repeatCount, hasMultipleOrigin) => {
        // Clone the fieldWrapper element the across the repeater.
        for( let repeatNo = 1; repeatNo <= repeatCount; repeatNo++) {
            const clonedField = fieldWrapper.clone().show().appendTo(fieldWrapper.parent());

            const uniqCloneIdentifier = hasMultipleOrigin ? `${originEl.data('optionid')}${ppomCfr.CONST_MULTIPLE_ORIGIN_IDENTIFIER}${repeatNo}` : `${repeatNo}`;

            // Handle magic tag
            handleMagicTag(clonedField, {
                'repeatNumber': repeatNo,
                'optionTitle': originEl.data('label')
            });

            clonedField.attr('data-clone_origin_group', originGroup);

            clonedField.addClass(CLONED_FIELD_INDICATOR_CLASS);

            // Add __clone suffix to needed stuff.
            rename_ids(clonedField, uniqCloneIdentifier, ()=>{});
        }
    }

    // Setup listener for the origin of the repeater.
    const setupListener = (repeaterDataName, hasMultipleOrigin) => {
        $(`input[data-data_name="${repeaterDataName}"]`).on('input', (e)=>{
            const originGroup = e.target.id;

            const originEl = $(e.target);

            // Reset the current cloned fields of the data name.
            $(`.ppom-field-wrapper[data-clone_origin_group="${originGroup}"]`).remove();

            // Represents the fieldWrapper clones will be how many.
            const repeater = e.target.value;

            const fields = $(`.ppom-field-wrapper:not(.${CLONED_FIELD_INDICATOR_CLASS}) .ppom-input[data-cfr_origin="${repeaterDataName}"]`);

            fields.each((index, field)=>{
                const jFieldWrapper = $(field).closest('.ppom-field-wrapper');
                clone(originEl, originGroup, jFieldWrapper, repeater, hasMultipleOrigin);
            });
        });
    }

    $(document).ready(() => {
        const originFields = {};
        const MULTIPLE_ORIGIN_FIELDS = ['quantities', 'qtypack']; // which fields has multiple origin.

        $('.ppom-wrapper .ppom-field-wrapper').each((i, fieldWrapper) => {
            const jFieldWrapper = $(fieldWrapper);
            const repeaterOriginDataName = jFieldWrapper.find('.ppom-input').data('cfr_origin');

            if( ! repeaterOriginDataName ) {
                return;
            }

            // Hide the the first field.
            jFieldWrapper.hide();

            const originInput = ppom_input_vars.ppom_inputs.find(field=>field.data_name===repeaterOriginDataName);

            if( ! originInput ) {
                return;
            }

            if( ! originFields.hasOwnProperty(repeaterOriginDataName) ) {
                originFields[repeaterOriginDataName] = {
                    hasMultipleOrigin: MULTIPLE_ORIGIN_FIELDS.includes( originInput.type )
                }
            }
        });

        Object.keys(originFields).forEach( (originDataName) => {
            const args = originFields[originDataName];
            setupListener(originDataName, args.hasMultipleOrigin);
        });
    });
})(jQuery);