<?php
/**
 * Handles addon loading
 */

namespace PPOM_Pro;

/**
 * Addon Loader
 */
class Addon_Loader {
	/**
	 * Holds the addons.
	 *
	 * @var string[]
	 */
	private $addons = [
		'Collapse',
		'Emojis',
		'Phone',
		'Repeater',
		'FieldsPopup',
		'CartEdit',
		'Chained',
		'ConditionalMeta',
		'Domain',
		'EnquiryForm',
		'FontsPicker',
		'Texter',
		'TextCounter',
		'FixedPrice',
		'SelectQty',
		'ImageDropdown',
		'Superlist',
		'OptionQty',
		'QuantitiesPack',
		'Switcher',
		'VariationMatrix',
		'BulkQuantity',
		'ConditionalFieldRepeater'
	// Requires composer dependency with a lot of fonts
	// 'Pdf',

	// Field doesn't appear. Works when importing meta from demo though.
	// 'EventCalendar',

	// Couldn't make it work.
	// 'VariationQuantity',

	// This seems to just add an SVG on the product page
	// 'Svg',

	// Doesn't work properly.
	// 'FancyCropper',

	// Added but didn't test with uploadcare.
	// 'UploadCare',

	// Dropped this as it makes no sense.
	// 'ImageChanger',

	// Doesn't work properly.
	// 'BulkTeams',

	// Has same code as select quantity. No ideea if this ever worked.
	// 'Autocomplete'
	];

	/**
	 * Constructor.
	 */
	public function __construct() {
		$this->load_addons();
	}

	/**
	 * Load addons.
	 *
	 * @return void
	 */
	private function load_addons() {
		foreach ( $this->addons as $addon ) {
			$addon_class = 'PPOM_Pro\\Addons\\' . $addon . '\\' . $addon;

			if ( ! class_exists( $addon_class ) ) {
				continue;
			}

			if ( ! is_subclass_of( $addon_class, Abstract_Addon::class ) ) {
				continue;
			}

			new $addon_class( $addon );
		}
	}

}
