<?php
/**
 * Plugin Name: PPOM Addon - Fields Collapse
 * Description: An addon to PPOM
 * Version: 1.1
 * Text Domain: ppom-addon-collapse
 * License: GPL2
 */

namespace PPOM_Pro\Addons\Collapse;

use PPOM_Pro\Abstract_Addon;

class Collapse extends Abstract_Addon {

	function run_hooks() {
		// Load action for input scripts
		add_action( 'ppom_hooks_inputs', array( $this, 'hook_input_scripts' ), 10, 2 );

		// File path
		add_filter( 'nm_input_class-collapse', array( $this, 'addon_path_collapse' ), 10, 2 );

		// Loading all input in PRO
		add_filter( 'ppom_all_inputs', array( $this, 'load_addon' ), 10, 2 );

		// PPOM Settings Filter
		add_filter( 'ppom_settings_data', array( $this, 'collapse_setting' ), 10, 1 );

		// plugin page setting hook
		$ppom_basename = plugin_basename( __FILE__ );
		add_filter( "plugin_action_links_{$ppom_basename}", array( $this, 'ppom_collapse_settings_link' ) );

		// ppom tabs control
		add_filter( 'ppom_fields_tabs_show', array( $this, 'tabs_render' ), 10, 2 );
	}


	/*
	**============= Load scripts ================
	*/
	function hook_input_scripts( $field, $data_name ) {

		if ( $field['type'] != 'collapse' ) {
			return '';
		}

		$localize_vars = array();

		$multiple_collapse_open = ppom_get_option( 'ppom-colapse-multiple-open' );
		$hide_collapse_arrow    = ppom_get_option( 'ppom-colapse-hide-arrow' );
		$icon_position          = ppom_get_option( 'ppom-colapse-icon-position' );
		$text_color             = ppom_get_option( 'ppom-colapse-text-color' );
		$bg_color               = ppom_get_option( 'ppom-colapse-bg-color' );
		$open_text_color        = ppom_get_option( 'ppom-colapse-open-text-color' );
		$open_bg_color          = ppom_get_option( 'ppom-colapse-open-bg-color' );
		$allow_nextprev         = ppom_get_option( 'ppom-collapse-nextprev' );

		if ( $field['type'] == 'collapse' ) {

			wp_enqueue_style( 'ppom-collapse', $this->addon_url . '/css/ppom-collapse-lib.css' );
			wp_enqueue_script( 'ppom-collapse-lib', $this->addon_url . '/js/ppom-collapse-lib.js', array( 'jquery' ), PPOM_PRO_VERSION, true );
			wp_enqueue_script( 'ppom-collapse', $this->addon_url . '/js/ppom-collapse.js', array( 'jquery' ), PPOM_PRO_VERSION, true );


			$localize_vars = array(
				'multiple_collapse_open' => $multiple_collapse_open,
				'data_name'              => $data_name,
				'hide_collapse_arrow'    => $hide_collapse_arrow,
				'icon_position'          => $icon_position,
				'text_color'             => $text_color,
				'bg_color'               => $bg_color,
				'open_text_color'        => $open_text_color,
				'open_bg_color'          => $open_bg_color,
				'allow_nextprev'         => $allow_nextprev,
			);

			wp_localize_script( 'ppom-collapse', 'ppom_collapse_vars', $localize_vars );
		}
	}


	/*
	**============= collapse inputs path load ================
	*/
	function addon_path_collapse( $path, $type ) {

		if ( file_exists( $this->addon_path . '/classes/input.collapse.php' ) ) {
			$path = $this->addon_path . "/classes/input.{$type}.php";
		}

		return $path;
	}


	/*
	**============= Loading all PRO inputs ================
	*/
	function load_addon( $inputs_array, $inputObj ) {

		// checking collapse addon is enable
		$inputs_array['collapse'] = $inputObj->get_input( 'collapse' );

		return $inputs_array;
	}

	/*
	**============= Collapse settings meta ================
	*/
	function collapse_setting( $ppom_settings ) {

		// Settings meta
		$ppom_settings[] = array(
			'title' => __( 'PPOM Collapse Settings', 'ppom-pro' ),
			'type'  => 'title',
			'desc'  => '',
			'id'    => 'ppom_collapse_id',
		);

		$ppom_settings[] = array(
			'title'   => __( 'Multiple Collapse Open', 'ppom-pro' ),
			'type'    => 'checkbox',
			'label'   => __( 'Button', 'ppom-pro' ),
			'default' => '',
			'id'      => 'ppom-colapse-multiple-open',
			'desc'    => __( 'Multiple collapse can be open at any given time.', 'ppom-pro' ),
		);

		$ppom_settings[] = array(
			'title'   => __( 'Hide Headline Arrow', 'ppom-pro' ),
			'type'    => 'checkbox',
			'label'   => __( 'Button', 'ppom-pro' ),
			'default' => '',
			'id'      => 'ppom-colapse-hide-arrow',
			'desc'    => __( 'Hide arrow down under headline.', 'ppom-pro' ),
		);

		$ppom_settings[] = array(
			'title'    => __( 'Collapse Icon Position', 'ppom-pro' ),
			'type'     => 'select',
			'label'    => __( 'Button', 'ppom-pro' ),
			'default'  => 'right',
			'options'  => array(
				'left'  => 'Left',
				'right' => 'Right',
			),
			'id'       => 'ppom-colapse-icon-position',
			'desc'     => __( 'Collapse icon position control.', 'ppom-pro' ),
			'desc_tip' => true,
		);

		$ppom_settings[] = array(
			'title'    => __( 'Collapse Text Color', 'ppom-pro' ),
			'type'     => 'color',
			'label'    => __( 'Button', 'ppom-pro' ),
			'default'  => '',
			'id'       => 'ppom-colapse-text-color',
			'desc'     => __( 'Change collapse text color.', 'ppom-pro' ),
			'desc_tip' => true,
		);

		$ppom_settings[] = array(
			'title'    => __( 'Collapse BG Color', 'ppom-pro' ),
			'type'     => 'color',
			'label'    => __( 'Button', 'ppom-pro' ),
			'default'  => '',
			'id'       => 'ppom-colapse-bg-color',
			'desc'     => __( 'Change collapse background color.', 'ppom-pro' ),
			'desc_tip' => true,
		);

		$ppom_settings[] = array(
			'title'    => __( 'Collapse Open Text Color', 'ppom-pro' ),
			'type'     => 'color',
			'label'    => __( 'Button', 'ppom-pro' ),
			'default'  => '',
			'id'       => 'ppom-colapse-open-text-color',
			'desc'     => __( 'Change collapse open text color.', 'ppom-pro' ),
			'desc_tip' => true,
		);

		$ppom_settings[] = array(
			'title'    => __( 'Collapse Open BG Color', 'ppom-pro' ),
			'type'     => 'color',
			'label'    => __( 'Button', 'ppom-pro' ),
			'default'  => '',
			'id'       => 'ppom-colapse-open-bg-color',
			'desc'     => __( 'Change collapse open background color.', 'ppom-pro' ),
			'desc_tip' => true,
		);
		$ppom_settings[] = array(
			'title'   => __( 'Allow Next/Prev', 'ppom-pro' ),
			'type'    => 'checkbox',
			'label'   => __( 'Button', 'ppom-pro' ),
			'default' => '',
			'id'      => 'ppom-collapse-nextprev',
			'desc'    => __( 'Allow user to show/hide collapse using prev/next button.', 'ppom-pro' ),
		);

		$ppom_settings[] = array(
			'type' => 'sectionend',
			'id'   => 'ppom_collapse_id',
		);


		return $ppom_settings;
	}


	function load_settings() {

		$settings = array(
			'ppom-colapse-multiple-open'   => array(
				'type'  => 'checkbox',
				'title' => __( 'Multiple Collapse Open', 'ppom-pro' ),
				'desc'  => __( 'Multiple collapse can be open at any given time.', 'ppom-pro' ),
			),
			'ppom-colapse-hide-arrow'      => array(
				'type'  => 'checkbox',
				'title' => __( 'Hide Headline Arrow', 'ppom-pro' ),
				'desc'  => __( 'Hide arrow down under headline.', 'ppom-pro' ),
			),
			'ppom-colapse-icon-position'   => array(
				'type'    => 'select',
				'title'   => __( 'Collapse Icon Position', 'ppom-pro' ),
				'default' => 'right',
				'options' => array(
					'left'  => 'Left',
					'right' => 'Right',
				),
				'desc'    => __( 'Collapse icon position control.', 'ppom-pro' ),
			),
			'ppom-colapse-text-color'      => array(
				'type'  => 'color',
				'title' => __( 'Collapse Text Color', 'ppom-pro' ),
				'desc'  => __( 'Change collapse text color.', 'ppom-pro' ),
			),
			'ppom-colapse-bg-color'        => array(
				'type'  => 'color',
				'title' => __( 'Collapse BG Color', 'ppom-pro' ),
				'desc'  => __( 'Change collapse background color.', 'ppom-pro' ),
			),
			'ppom-colapse-open-text-color' => array(
				'type'  => 'color',
				'title' => __( 'Collapse Open Text Color', 'ppom-pro' ),
				'desc'  => __( 'Change collapse open text color.', 'ppom-pro' ),
			),
			'ppom-colapse-open-bg-color'   => array(
				'type'  => 'color',
				'title' => __( 'Collapse Open BG Color', 'ppom-pro' ),
				'desc'  => __( 'Change collapse open background color.', 'ppom-pro' ),
			),
			'ppom-collapse-nextprev'       => array(
				'type'  => 'checkbox',
				'title' => __( 'Allow Next/Prev', 'ppom-pro' ),
				'desc'  => __( 'Allow user to show/hide collapse using prev/next button.', 'ppom-pro' ),
			),
		);

		$panels = array(
			'ppom_addon_collapse' => array(
				'id'          => 'ppom_addon_collapse',
				'title'       => __( 'Collapse', 'ppom-pro' ),
				'desc'        => '',
				'icon'        => '',
				'is_sabpanel' => true,
				'active'      => 'yes',
			),
		);

		$tabs = array(
			'ppom_addons' =>
				array(
					'tab_id' => 'ppom_addons',
					'title'  => 'Addons',
					'enable' => true,
				),
		);

		PPOMSETTINGS()->register_tabs( $tabs )->register_panel( 'ppom_addons', $panels )->register_setting( 'ppom_addon_collapse', $settings );
	}


	/*
	**========== Settings Link On Plugin Page ===========
	*/
	function ppom_collapse_settings_link( $links ) {

		$wc_settings_page = admin_url( 'admin.php?page=wc-settings' );
		$ppom_pdf_url     = add_query_arg( array( 'tab' => 'ppom_settings' ), $wc_settings_page );

		$ppom_links   = array();
		$ppom_links[] = sprintf( __( '<a href="%s">Settings</a>', 'ppom-pro' ), esc_url( $ppom_pdf_url ) );


		foreach ( $ppom_links as $link ) {

			array_push( $links, $link );
		}

		return $links;
	}


	/*
	**============= Show variation tabs ================
	*/
	function tabs_render( $tabs, $fields_type ) {

		if ( $fields_type == 'collapse' ) {

			foreach ( $tabs as $tab_id => $tab_meta ) {
				if ( $tab_id == 'condition_tab' ) {

					$tabs['condition_tab'] = array(
						'label'        => __( 'Conditions', 'ppom-pro' ),
						'class'        => array( 'ppom-tabs-label', 'ppom-condition-tab-js' ),
						'field_depend' => array( 'all' ),
						'not_allowed'  => array( 'collapse' ),
					);
				}
			}
		}

		return $tabs;
	}
}
