"use strict";
jQuery(function($){


    /*************************************
    *   PPOM Calendar Event Addon JS     *
    **************************************/
    
    
    PPOMWrapper.on('change', '.ppom-eventcalendar', function(e){
     
        ppom_update_option_prices();
   });


    /*
    **============= Get PPOM inputs fields meta ================ 
    */
    var fields_meta = ppom_input_vars.field_meta;


    /*
    **============= Loop PPOM inputs fields meta ================ 
    */
    $.each(fields_meta, function (index, value) {
        
        var fields_type = value.type;
        
        /*============= Event calendar type check =============*/
        if ( fields_type == 'eventcalendar' ) {

            var data_name = value.data_name;
            
            /*============= localize script ================*/
            var settingObj = window["ppom_ce_"+data_name+"_vars"];
            
            var calendar_meta      = settingObj.ce_meta;
            var disabled_weekends  = settingObj.disabled_weekends;
            var ticket_message     = settingObj.ticket_message;
            var date_formate       = settingObj.date_formate;
            var tooltip_ticket_msg = settingObj.tooltip_ticket_msg;
            var simple_ticket_msg  = settingObj.simple_ticket_msg;
            var startofday         = settingObj.startofday;
            var price_msg          = settingObj.price_msg;
            var hide_baseqty       = settingObj.hide_baseqty;
            var tickets_vaiation   = settingObj.tickets_vaiation;
            var global_price       = settingObj.global_price;
            var disabled_global_price = settingObj.disabled_global_price;
            
            /*============= Built json ================*/
            var eventDates   = {};
            var tickets_arr  = {};
            
            /*=========== Selected dates through loop ===========*/
            $.each(calendar_meta, function (index, value) {

                var tickets = value.ticket;
                var price   = value.price;
                
                if (disabled_weekends == 'on') {

                    var index = is_weekend(index, eventDates);
                }

                eventDates[index]  =  index;
                tickets_arr[index] = tickets;
            });

            // render Calendar
            datecalendar_init(data_name, date_formate, ticket_message, eventDates, tickets_arr, startofday, tooltip_ticket_msg, simple_ticket_msg, tickets_vaiation, disabled_global_price, global_price, price_msg);
        
            // enabled tooltip after select dates
            show_ticket_msg_tooltip(tooltip_ticket_msg, data_name);

            // hide QTY box
            if (hide_baseqty == 'yes') {
                $('form.cart input[name="quantity"]').parent().css('display', 'none');
            }
        }
    });


    /*
    **============= Jquery UI DatePicker ================ 
    */
    function datecalendar_init(data_name, date_formate, ticket_message, eventDates, tickets_arr, startofday, tooltip_ticket_msg, simple_ticket_msg, tickets_vaiation, disabled_global_price, global_price, price_msg){

        // get current date for disabled past dates from calendar
        var current_date = new Date();
        
        $('#ppom-datepicker-'+data_name).datepicker({

            dateFormat: date_formate,
            firstDay: startofday,
            minDate: current_date,
            beforeShowDay: function( date ) {
                
                var formatted_date = jQuery.datepicker.formatDate('yy-mm-dd', date);
                var selected_dates = eventDates[formatted_date];
                var tickets        = tickets_arr[formatted_date];

                var tickets_display_msg = '';
                if (tooltip_ticket_msg == 'on') {
                    tickets_display_msg = ticket_message.replace("%TICKET%", tickets);
                }

                if( selected_dates) {
                    return [true, "eventcalendar-active-date", tickets_display_msg];
                } else {
                    return [false, '', ''];
                }
            },

            onSelect: function(event_date, inst) {

                var calendar_id    = inst.id;
                var dateAsObject   = $(this).datepicker( 'getDate' );
                var formatted_date = jQuery.datepicker.formatDate('yy-mm-dd', dateAsObject);
                var tickets        = tickets_arr[formatted_date];

                var tickets_display_msg = '';
                if (simple_ticket_msg == 'on') {
                    tickets_display_msg = ticket_message.replace("%TICKET%", tickets);
                }

                // enabled tooltip after select dates
                show_ticket_msg_tooltip(tooltip_ticket_msg, data_name);

                // remove selected date tooltip box from body
                $('#ppom_tooltip').remove();

                // create selected date box for information and select tickets
                if (disabled_global_price != 'on') {
                    ppom_eventcalendar_create_simple_ticket_html( event_date, tickets, data_name, calendar_id, formatted_date, tickets_display_msg, global_price, price_msg );
                }else{
                    ppom_eventcalendar_create_tickets_variation_html(event_date, tickets, data_name, calendar_id, formatted_date, tickets_display_msg, tickets_vaiation , price_msg);
                }

            },

            onChangeMonthYear: function(){

                // enabled tooltip after select dates
                show_ticket_msg_tooltip(tooltip_ticket_msg, data_name)
            },
        });
    }


    /*
    **============= Ticket Message Over The Tooltip ================ 
    */
    function show_ticket_msg_tooltip(tooltip_ticket_msg, data_name){

        if (tooltip_ticket_msg == 'on') {
            setTimeout(function() {
                $('#ppom-datepicker-'+data_name+' .ui-datepicker-calendar .eventcalendar-active-date').attr('data-ppom-tooltip', 'ppom_tooltip'); // Add custom data here
            }, 0);
        }
    }
   
    
    /*
    **============= Create Selected Date Box ================ 
    */
    function ppom_eventcalendar_create_simple_ticket_html( event_date, tickets, dataname, calendar_id, formatted_date, tickets_display_msg, global_price, price_msg) {

        var div       = $("#"+ calendar_id).parent();
        var container = div.find('.ppom-eventcalendar-append');
        var the_id    = 'ppom-'+dataname+'-'+formatted_date+'-rm';
        var html = '';

        // display price with wc formatted
        var ticket_price_msg = '';
        ticket_price_msg = display_eventcalendar_price_with_formatted(global_price, price_msg);
        
        // remove/reset
        $("#"+the_id).remove();
        
        html += '<div class="form-row ppom-eventcalendar-selecteddates-box" id="'+the_id+'">';
            
            html += '<div class="col-md-5 col-sm-5">';
                html += '<span class="ppom-eventcalendar-date">'+event_date+'</span>';
            html += '</div>';

            html += '<div class="col-md-4 col-sm-4">';
                
                html += '<span class="ppom-eventcalendar-info">'+ticket_price_msg+'</span>';
                
                if (tickets_display_msg != '') {

                    html += '<span class="ppom-eventcalendar-ticket-msg">'+tickets_display_msg+'</span>';
                }
            html += '</div>';

            html += '<div class="col-md-2 col-sm-2">';
                html += '<span class="ppom-eventcalendar-qty"><input class="ppom-eventcalendar" type="number" name="ppom[fields]['+dataname+']['+formatted_date+'][Simple]" max="'+tickets+'" min="0" data-label="'+event_date+'" data-price="'+global_price+'"></span>';
            html += '</div>';

            html += '<div class="col-md-1 col-sm-1">';
                html += '<span class="ppom-eventcalendar-rm-date"><i class="fa fa-times" aria-hidden="true"></i></span>';
            html += '</div>';
            
        html += '</div>';

        $(html).appendTo(container); 

        // reset quantity to zero
        $("#"+the_id).find('input[type="number"]').val('').change();
    }


    function ppom_eventcalendar_create_tickets_variation_html(event_date, tickets, dataname, calendar_id, formatted_date, tickets_display_msg, tickets_vaiation, price_msg){

        var div       = $("#"+ calendar_id).parent();
        var container = div.find('.ppom-eventcalendar-append');
        var the_id    = 'ppom-'+dataname+'-'+formatted_date+'-rm';
        var ticket_price_msg = '';
        var html = '';

        // remove/reset
        $("#"+the_id).remove();

        html += '<div class="ppom-eventcalendar-variation-table" id="'+the_id+'">';

            html += '<table class="table table-bordered">';
                html += '<thead>';
                    html += '<tr>';
                        html += '<th colspan="2">'+event_date
                            if (tickets_display_msg != '') {

                                html += '<span class="ppom-eventcalendar-ticket-msg">'+tickets_display_msg+'</span>';
                            }
                            html += '<span class="ppom-eventcalendar-rm-date"><i class="fa fa-times" aria-hidden="true"></i></span>';
                        html +'</th>';
                    html += '</tr>';
                html += '</thead>';
                html += '<tbody>';
        
                    $.each(tickets_vaiation, function(variation_name, variation_meta){
                        html += '<tr>';
                            var ticket_variation_price = variation_meta.price;
                            var ticket_variation_id    = variation_meta.option_id;

                            var label_msg = event_date +' ('+ variation_name + ')';
                            // display price with wc formatted
                            ticket_price_msg = display_eventcalendar_price_with_formatted(ticket_variation_price, price_msg);

                            html += '<th>';
                                html += '<label class="ppom-eventcalendar-date"> '+variation_name;
                                    html += '<span class="ppom-eventcalendar-info">'+ticket_price_msg+'</span>';
                                html += '</label>';
                            html += '</th>';
                            html += '<td>';
                                html += '<input class="ppom-eventcalendar" type="number" name="ppom[fields]['+dataname+']['+formatted_date+']['+variation_name+']" max="'+tickets+'" min="0" data-label="'+label_msg+'" data-price="'+ticket_variation_price+'">';
                            html += '</td>';
                        html += '</tr>';
                    }); 
                    
                html += '</tbody>';
            html += '</table>';

        html += '</div>';

        $(html).appendTo(container);

        // reset quantity to zero
        $("#"+the_id).find('input[type="number"]').val('').change();
    }


    /*
    **============= WC Price Formatted ================ 
    */
    function display_eventcalendar_price_with_formatted(ticket_price, price_msg){

        var formattde_price  = ppom_get_wc_price(ticket_price).html();
        var ticket_price_msg = price_msg.replace("%PRICE%", formattde_price);

        return ticket_price_msg;
    }


    /*
    **============= Disabled Weekends ================ 
    */
    function is_weekend(date1, eventDates){
        var dt = new Date(date1);
         
        if(dt.getDay() == 6 || dt.getDay() == 0){
            return delete eventDates[date1];
        }else{
            return date1;
        }
    }


    /*
    **============= Remove QTY Date Box ================ 
    */
    $(document).on('click', '.ppom-eventcalendar-rm-date', function(e){
         e.preventDefault();

        var selected_date = $(this).closest('.ppom-eventcalendar-selecteddates-box, .ppom-eventcalendar-variation-table');
        selected_date.find('input[type="number"]').val('').change();
        selected_date.remove();
    });

});