<?php
/**
 * Plugin Name: PPOM Addon: PDF
 * Description: PPOM ADDON - Print pdf of product meta.
 * Version: 1.2
 * License: GPLv2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: ppom-addon-pdf
 */

namespace PPOM_Pro\Addons\Pdf;

use PPOM_Pro\Abstract_Addon;

class Pdf extends Abstract_Addon {
	protected static $mpdf;

	function run_hooks() {
		// Show metaboxe on order template
		add_action( 'admin_init', array( $this, 'render_files_in_orders' ) );

		// admin action for pdf print
		add_action( 'admin_post_ppom_pdf_meta', array( $this, 'ppom_print_pdf_product_meta' ) );

		// PPOM Settings Filter
		add_filter( 'ppom_settings_data', array( $this, 'pdf_setting_show' ), 10, 1 );

		// plugin page setting hook
		// TODO: CHECK THIS PROBABLY REMOVE
		$ppom_basename = PPOM_PLUGIN_PATH;
		add_filter( "plugin_action_links_{$ppom_basename}", array( $this, 'ppom_pdf_settings_link' ) );

	}


	/* 
	**========== Create Metabox In Order Page =========== 
	*/
	function render_files_in_orders() {

		// PDF print box
		add_meta_box(
			'ppom_pdf_box',
			__( 'PPOM Print PDF', 'ppom-pro' ),
			array( $this, 'display_pdf_order_meta' ),
			'shop_order',
			'normal',
			'default'
		);
	}


	/*
	**========== Metabox Callback Function =========== 
	*/
	function display_pdf_order_meta( $order ) {
		wp_enqueue_style( 'ppom-pdf-css', $this->addon_url . '/assets/ppom-pdf.css' );

		$this->load_template(
			'pdf-box.php',
			array(
				'order_id' => $order->ID,
				'section'  => 'ppom_simple_view',
			) 
		);
	}


	/*
	**========== Print Callback function =========== 
	*/
	function ppom_print_pdf_product_meta() {
		$pdf_dir_path = ppom_get_dir_path( 'pdf_dir' );

		self::$mpdf = new \Mpdf\Mpdf(
			[
				'tempDir'             => $pdf_dir_path,
				'margin_left'         => 1,
				'margin_right'        => 1,
				'margin_top'          => 1,
				'setAutoTopMargin'    => 'stretch',
				'autoMarginPadding'   => 0,
				'setAutoBottomMargin' => 'stretch',
				'margin_bottom'       => 0,
				'margin_header'       => 0.4,
				'margin_footer'       => 0.4,
			]
		);

		$order_id = isset( $_REQUEST['order_id'] ) ? $_REQUEST['order_id'] : '';

		$disable_hf = ppom_get_option( 'ppom-pdf-disable-hf' );

		ob_start();
		$templates_name = 'pdf-box.php';
		$template_vars  = array(
			'order_id' => $order_id,
			'section'  => 'ppom_pdf_view',
		);
		$this->load_template( $templates_name, $template_vars );
		$html = ob_get_clean();

		$stylesheet = file_get_contents( $this->addon_url . '/assets/ppom-pdf.css' );

		if ( $disable_hf == 'no' ) {
			self::$mpdf->SetHTMLHeader( $this->pdf_header() );
			self::$mpdf->SetHTMLFooter( $this->pdf_footer() );
		}

		self::$mpdf->WriteHTML( $stylesheet, 1 );
		self::$mpdf->WriteHTML( $html, 2 );
		self::$mpdf->Output();
	}


	/*
	**========== PDF Header function ===========
	*/
	function pdf_header() {

		$html  = '';
		$html .= '<div style="text-align: center; background-color: #b4c1ff85;" class="ppom-pdf-header">';
		$html .= '<h1>' . get_bloginfo( 'name' ) . '</h1>';
		$html .= '</div>';

		return $html;
	}


	/*
	**========== PDF Footer function ===========
	*/
	function pdf_footer() {

		$html = '';

		$html  = '<div style="text-align: right; background-color: #b4c1ff85;">';
		$html .= '<h2 style="margin-right:3px;padding-bottom:3px;">' . get_bloginfo( 'url' ) . '</h2>';
		$html .= '</div>';

		return $html;
	}


	/*
	**============= PDF settings meta ================
	*/
	function pdf_setting_show( $ppom_settings ) {

		// Settings meta
		$ppom_settings[] = array(
			'title' => __( 'PPOM PDF Settings', 'ppom-pro' ),
			'type'  => 'title',
			'desc'  => '',
			'id'    => 'ppom_pdf_id',
		);

		$ppom_settings[] = array(
			'title'    => __( 'PDF Header', 'ppom-pro' ),
			'type'     => 'textarea',
			'label'    => __( 'Enable', 'ppom-pro' ),
			'default'  => '',
			'id'       => 'ppom-pdf-header-html',
			'desc'     => __( 'Add HTML code here for pdf header and add inline css for styling.', 'ppom-pro' ),
			'desc_tip' => true,
		);

		$ppom_settings[] = array(
			'title'    => __( 'PDF Footer', 'ppom-pro' ),
			'type'     => 'textarea',
			'label'    => __( 'Button', 'ppom-pro' ),
			'default'  => '',
			'id'       => 'ppom-pdf-footer-html',
			'desc'     => __( 'Add HTML code here for pdf footer and add inline css for styling.', 'ppom-pro' ),
			'desc_tip' => true,
		);

		$ppom_settings[] = array(
			'title'   => __( 'Disable Header/Footer', 'ppom-pro' ),
			'type'    => 'checkbox',
			'label'   => __( 'Button', 'ppom-pro' ),
			'default' => '',
			'id'      => 'ppom-pdf-disable-hf',
			'desc'    => __( 'Disable defualt header and footer section.', 'ppom-pro' ),
		);

		$ppom_settings[] = array(
			'type' => 'sectionend',
			'id'   => 'ppom_pdf_id',
		);


		return $ppom_settings;
	}


	function load_settings() {

		$settings = array(
			'ppom-pdf-header-html' => array(
				'type'    => 'textarea',
				'title'   => __( 'PDF Header', 'ppom-pro' ),
				'desc'    => __( 'Add HTML code here for pdf header and add inline css for styling.', 'ppom-pro' ),
				'tooltip' => true,
			),
			'ppom-pdf-footer-html' => array(
				'type'    => 'textarea',
				'title'   => __( 'PDF Footer', 'ppom-pro' ),
				'desc'    => __( 'Add HTML code here for pdf footer and add inline css for styling.', 'ppom-pro' ),
				'tooltip' => true,
			),
			'ppom-pdf-disable-hf'  => array(
				'type'  => 'checkbox',
				'title' => __( 'Disable Header/Footer', 'ppom-pro' ),
				'desc'  => __( 'Disable defualt header and footer section.', 'ppom-pro' ),
			),
		);

		$panels = array(
			'ppom_addon_pdf' => array(
				'id'          => 'ppom_addon_pdf',
				'title'       => __( 'PDF', 'ppom-pro' ),
				'desc'        => 'It will render the PDF settings.',
				'icon'        => '',
				'active'      => 'yes',
				'is_sabpanel' => true,
			),
		);

		$tabs = array(
			'ppom_addons' =>
				array(
					'title'  => 'Addons',
					'tab_id' => 'ppom_addons',
					'enable' => true,
				),
		);

		PPOMSETTINGS()->register_panel( 'ppom_addons', $panels )->register_setting( 'ppom_addon_pdf', $settings );
	}

	/*
	**========== Settings Link On Plugin Page ===========
	*/
	function ppom_pdf_settings_link( $links ) {

		$wc_settings_page = admin_url( 'admin.php?page=wc-settings' );
		$ppom_pdf_url     = add_query_arg( array( 'tab' => 'ppom_settings' ), $wc_settings_page );

		$ppom_links   = array();
		$ppom_links[] = sprintf( __( '<a href="%s">Settings</a>', 'ppom-pro' ), esc_url( $ppom_pdf_url ) );


		foreach ( $ppom_links as $link ) {
			$links[] = $link;
		}

		return $links;
	}
}
