<?php

namespace PPOM_Pro\Traits;

trait Meta_Utility {
	/**
	 * Returns the array index of the given field in the given fields group.
	 * Matches with the "data_name".
	 *
	 * @param  array $ppom_fields array of the fields
	 * @param  array $field a field. Some the properties (stock, ticket etc.) of the $field array can be different with the field one in the $ppom fields.
	 * @return int|false
	 */
	private function find_field_index_in_meta_group( $ppom_fields, $field ) {
		foreach ( $ppom_fields as $index => $field_meta ) {
			if ( $field_meta['data_name'] === $field['data_name'] ) {
				return $index;
			}
		}

		return false;
	}

	/**
	 * Replaces the saved PPOM field data.
	 * Generally that method is used in areas such as reduce stock processes when order completed, or reduce remain ticket in Event Calender module etc.
	 *
	 * @param  array $updated_field_meta Array of the updated PPOM Field.
	 * @return void
	 */
	public function update_meta_settings( $updated_field_meta ) {
		if ( ! isset( $updated_field_meta['ppom_id'] ) ) {
			return;
		}

		$product_id = null;

		$ppom = \PPOM_Meta::get_instance( $product_id );

		// Getting ppom_id inside save meta
		$ppom_id     = $updated_field_meta['ppom_id'];
		$ppom_fields = $ppom->get_fields_by_id( $ppom_id );

		$field_index = $this->find_field_index_in_meta_group( $ppom_fields, $updated_field_meta );

		if ( $field_index === false ) {
			return;
		}

		$ppom_fields[ $field_index ] = $updated_field_meta;

		ppom_admin_update_ppom_meta_only( $ppom_id, $ppom_fields );
	}
}
