<?php
/**
 * class FieldClone
 *
 * @package PPOM_Pro\Utility
 */
namespace PPOM_Pro\Utility;

use PPOM_Pro\Traits\StringUtility;

/**
 * Class FieldClone
 * That class is responsible for containing reusable utility methods that are to be used in Conditional Field Repeater, and Repeater Addons.
 */
class FieldClone {
    use StringUtility;

    /**
     * If a field data name contains the string that in the value; that represents that's a cloned field.
     */
    const CLONE_DATANAME_INDICATOR = '__clone_';

    /**
     * Do not allow initialization since the all methods are static.
     *
     * @return void
     */
    private function __construct(){}

    /**
     * Extract the Repeat Number from the data name
     *
     * @param  string $dataname
     * @return false|int If that's a cloned field returns the repeating No, otherwise returns false.
     */
    public static function get_clone_repeat_id($dataname) {
        $is_cloned_field = self::str_contains( $dataname, self::CLONE_DATANAME_INDICATOR );

        if( ! $is_cloned_field ) {
            return false;
        }

        return explode( self::CLONE_DATANAME_INDICATOR, $dataname )[1];
    }

    /**
     * Extract original dataname from the cloned dataname
     *
     * @param  string $dataname Data name that contains clone indicator string.
     * @return string Original data name (can be matched with the field data name in a PPOM field group.)
     */
    public static function get_original_dataname($dataname){
        return self::remove_specific_suffix($dataname, self::CLONE_DATANAME_INDICATOR);
    }
}